/*
 * Nom du fichier: RemonteTable_Uno_Bouvet.ino
 * Auteur: Wyldix (contact à travers le forum metabricoleur.com)
 * Date de création: 02.04.2015
 * Date de modification: 24.12.2016
 * Version: 1.2
 * Description: Programme qui permet de déplacer la table d'une raboteuse à la hauteur voulue
 */

// Librairies/fichiers annexes
#include <Keypad.h>
#include <Wire.h>
#include <AccelStepper.h>
#include <math.h>
#include "config.h"

// Constantes définies par l'utilisateur dans le fichier "config.h". Ne pas modifier ici!
#define HHOME (long)(HAUTEUR_HOME*10)   // Hauteur de la table au repos      
#define HMIN (HAUTEUR_MIN*10)        		// Hauteur minimale de la table en 1/10 mm
#define HMAX (HAUTEUR_MAX*10)        		// Hauteur maximale de la table en 1/10 mm
#define PASPARMM (long)((NBR_PAS_MOTEUR*MICRO_PAS*DENTS_POULIE_MANIVELLE)/(10*PAS_VIS*DENTS_POULIE_MOTEUR))      // Nombre de pas du moteur pour que la table monte de 1/10 mm
#define ACCEL (ACCELERATION*PASPARMM*10)    // Accélération en mm/s^2 (200 = pas/s^2; d'où la multiplication par PASPARMM)
#define ACCELSTOP (10*ACCEL)         		    // Décélération en cas d'arrêt d'urgence
#define VITESSE ((VITESSE_MAX/6)*PASPARMM)  // Vitesse maximale de montée de la table en mm/min (ici 500; c'est le premier nombre qu'il faut changer)

// Nombre de lignes et de colonnes du clavier
const byte LIG = 4;       // 4 lignes
const byte COL = 4;       // 4 colonnes

// Tableau contenant la répartition des touches du clavier numérique
char numTouches[LIG][COL] = {     // F = Fonctions
  { 1 , 2 , 3 ,'F'},              // I = Incrément/absolu
  { 4 , 5 , 6 ,'I'},              // S = Start
  { 7 , 8 , 9 ,'S'},              // P = stoP
  {'H', 0 ,'C','P'}               // H = Home
};                                // C = Clear

// Connections correspondantes aux lignes et colonnes du clavier
byte rowPins[LIG] = {Lig1Pin, Lig2Pin, Lig3Pin, Lig4Pin};   
byte colPins[COL] = {Col1Pin, Col2Pin, Col3Pin, Col4Pin};

// Création de deux claviers, un avec les nombres et l'autre avec les fonctions
Keypad numClav = Keypad(makeKeymap(numTouches), rowPins, colPins, LIG, COL);

// Définition d'un moteur pas à pas (control par driver avec step/dir), ajout du Enable possible avec setEnablePin()
// Step: une transition bas->haut signifie un pas (changement possible avec setPinsInverted())
// Dir:  niveau logique haut signifie rotation sens horaire
AccelStepper moteur(1, StepPin, DirPin);

// Variables globales
long hConsigne = 0;                  // Hauteur entrée par l'utilisateur 
long hActuelle = HHOME;              // Variable contenant en permanence la hauteur actuelle de la table     
static byte kpadState;               // Etat de la touche (PRESSED, HOLD, RELEASED, repectivement appuyée, maintenue ( > 2s dans notre cas), relâchée)
char key;                            // Touche appuyée
long precedentMillis = 0;            // Sauvegarde de la dernière fois que l'écran a été rafraîchit
boolean moteurTourne = false;        // Flag signalant lorsque le moteur tourne
boolean moteurStop = false;          // Flag signalant un arrêt d'urgence

// Fonction exécutée au démarrage -> Initialisation de tous les périphériques nécessaires (com. avec le PC, afficheurs, clavier,...)
void setup(void)
{
  Serial.begin(9600);                               // Moniteur sériel pour le débogage (liaison avec le PC)
  
  // Initialisation des afficheurs et effacement du contenu de ces derniers
  Wire.begin();                                     // Initialisation du bus I2C
  
  Wire.beginTransmission(AFFSUP_ADRESSE);           // Début de la communication avec le premier afficheur
  Wire.write(0x7A);                                 // Commande pour le contrôle de la luminosité
  Wire.write((byte) LUMINOSITE1);                   // Valeur de la luminosité de l'afficheur
  Wire.write('v');                                  // Effacement du contenu de l'afficheur (par sécurité)
  Wire.endTransmission();                           // Fin de la communication avec l'afficheur
  
  Wire.beginTransmission(AFFINF_ADRESSE);           // Début de la communication avec le second afficheur
  Wire.write(0x7A);                                 // Commande pour le contrôle de la luminosité
  Wire.write((byte) LUMINOSITE2);                   // Valeur de la luminosité de l'afficheur
  Wire.write('v');                                  // Effacement du contenu de l'afficheur (par sécurité)
  Wire.endTransmission();                           // Fin de la communication avec l'afficheur
  
  // Initialisation du moteur
  moteur.setCurrentPosition(hActuelle * PASPARMM);  // Hauteur actuelle au démarrage (Home)
  moteur.setMaxSpeed(VITESSE);                      // Initialisation de la vitesse maximale et de l'accélération du moteur pas à pas
  moteur.setAcceleration(ACCEL);
  moteur.setMinPulseWidth(100);                     // Impulsions de 100 us
  moteur.setPinsInverted(INVDIR, false, false);     // Inversion possible de dir (step et inable restent à leur valeur par défaut)
    
  // Initialisation des événements relatifs au clavier
  numClav.addEventListener(keypadEvent_num);        // Ajout d'un événement pour le clavier numérique
  numClav.setHoldTime(2000);                        // Par défaut 1000 ms (touche appuyée), maintenant 2000 ms
  
   // Affichage des valeurs à l'allumage
  affDec(moteur.currentPosition() / PASPARMM, AFFSUP);
  affDec(hConsigne, AFFINF); 
}

// Boucle sans fin -> tourne s'exécute aussi longtemps que la carte est sous tension
void loop(void)
{  
  moteurRun();
} 

// Evénement concernant le clavier numérique
void keypadEvent_num(KeypadEvent key){
    kpadState = numClav.getState();
    swOnState(key);
} 


// Fonction qui stoppe le moteur -> arrêt d'urgence (déclenchée en appuyant sur la touche "stop")
void stopMoteur() {
  if(moteurTourne) {
    moteur.setAcceleration(ACCELSTOP);      // Augmentation de l'accélération pour un freinage rapide
    moteur.stop();                          // Envoie de la position d'arrêt
    moteurStop = true;                      // Pour signaler à la boucle principale l'urgence... 
  } 
}

void swOnState(char key) {
    switch(kpadState) {
      case PRESSED: {
        if(key == 'C') {   // "Clear"
          hConsigne = 0;  
        }
        else if((key >= 0) && (key <= 9)) {       // Chiffre 
            hConsigne = (hConsigne * 10) + key;
        }
        else if(key == 'S') {    // "Start"
          // Pour éviter de dépasser les limites de la machine, si une des limites est dépassée, la valeur est limitée
          // et il faut confirmer (ou modifier) en réappuyant sur "Start"
          if(hConsigne > HMAX) {
            hConsigne = HMAX; 
          } 
          else if(hConsigne < HMIN) {
            hConsigne = HMIN;
          }
            else {
              // En appuyant sur Start, le moteur se déplace jusqu'à la position voulue
              moteur.moveTo(hConsigne * PASPARMM);  // Position souhaitée
              moteurTourne = true;
            }
        }
        else if(key == 'P') {    // "Stop"    
            stopMoteur();                                        // En appuyant sur Stop, le moteur s'arrête
        }
        break;
      } 
      default: break;
    }

    affDec(hConsigne, AFFINF);  // Affichage inférieur
}

void affDec(unsigned int valeur, byte afficheur) {
  boolean flagDec = false;          // Pour afficher les zéros...
  
  // Choix du bon afficheur en fonction du paramètre "afficheur"
  if(afficheur == AFFSUP) Wire.beginTransmission(AFFSUP_ADRESSE);
  else Wire.beginTransmission(AFFINF_ADRESSE);
  
  // Si le choix de l'afficheur est correct, mais la valeur à afficher est trop grande, rien ne se passe...
  if(valeur < 10000) {
    Wire.write('v');                // Effacement de l'affichage
    Wire.write(0x77);               // Commande pour allumer les points décimaux
    Wire.write(0x04);               // Affichage du point décimal
    
    if(valeur > 999) {
      Wire.write(valeur / 1000);    // Envoie du digit de gauche
      if((valeur %= 1000) < 100)    // Enlever le digit de gauche du nombre qu'on souhaite afficher
        flagDec = true;             // Si le digit suivant est un zéro, agiter le drapeau!
    }
    else Wire.write(0x10);          // Envoie d'un caractère vide (digit éteint)
    
    if((valeur > 99) || flagDec) {
      Wire.write(valeur / 100);     // Prochain digit...
      if((valeur %= 100) < 10)
        flagDec = true;
    }
    else Wire.write(0x10);          // Envoie d'un caractère vide (digit éteint)
    
    Wire.write(valeur / 10);
    valeur %= 10;
    
    Wire.write(valeur);             // Envoie du digit de droite
  }
  Wire.endTransmission();           // Fin de la transmission
}

void moteurRun() {
  unsigned long actuelMillis = millis();        // Temps écoulé depuis l'allumage de la carte Arduino
             
  key = numClav.getKey();						// Lecture de la touche du clavier pressée
    
  if(moteurTourne) {
    if(moteurStop) {
      moteur.runToPosition();
      moteurStop = false;
      moteur.setAcceleration(ACCEL);
    }
    
    hActuelle = round(moteur.currentPosition() / PASPARMM);      // Position actuelle du moteur (donc de la table)
    
    if(moteur.distanceToGo() != 0) {
      moteur.run();
    }
    else {
      moteurTourne = false;
      hConsigne = 0;
      affDec(hActuelle, AFFSUP);
      affDec(hConsigne, AFFINF);
    }
    
     // Rafraîchissement de l'écran seulement si le moteur tourne (autrement pas de variation de la hauteur actuelle!)
     if(actuelMillis - precedentMillis > INTERVAL) {
       precedentMillis = actuelMillis;            // Sauvegarde du temps du dernier rafraîchissement de l'écran
       affDec(hActuelle, AFFSUP);                 // Affichage de la hauteur actuelle
     } 
  }
}



